#!/bin/ruby
# dspg by Daniel Roggen, 2009
# 
#

require 'ftools'
require 'fileutils'
include FileUtils

###############################################################################
# Banner
###############################################################################
#
def Banner
	print "DSPG - Static Page Generator\n"
	print "   by Daniel Roggen, 2009\n"

end

###############################################################################
# Error
###############################################################################
#
def Syntax
	print "\n"
	print "Converts a template, configuration file and html content pages into\n"
	print "a linked set of pages with a sidebar navigation menu.\n"
	print "\n"
	print "A template forms the basis of the generated pages.\n"
	print "Inside the template specific keywords are replaced by:\n"
	print " (i) a programmatically generated static menu navigation sidebar\n"
	print " (ii) the content of the HTML content files\n"
	print " (iii) the page title.\n"
	print "\n"
	print "Syntax: dspg src_dir target_dir [copy]\n"
	print "        src_dir:    directory containing the source files.\n"
	print "                    It must include:\n"
	print "                       'config' file: configuration\n"
	print "                       'template' file: layout template\n"
	print "                       '*.html' files: html files that will be converted\n"
	print "        target_dir: existing directory receiving the generated files.\n"
	print "        copy:       if present will copy the content of directory src_dir into target_dir\n"
	print "\n"
	print "Format of the configuration file 'config': \n"
	print "  - One entry per line.\n"
	print "  - Lines with '#' are comments\n"
	print "  - Entry format: [---]Page title[;page.html]\n"
	print "  - Optional leading dashes: header level. No dash=highest level\n"
	print "  - Page title: text appearing in the navigation menu\n"
	print "  - page.html: optional html file with the page content.\n"
	print "               When not specified page of a sub-header is shown.\n"
	print "\n"
	print "The following keywords in the template are replaced:\n"
	print "DSPG-INCLUDE-SIDEMENU:  replaced by a programmatically generated sidebar\n"
	print "                        Sidebar entries are of the form: <p class=\"shX_[active|inactive]\">Page title...<a href=...\n"
	print "                        The styles shX_[active|inactive] must be defined in a style sheet\n"
	print "DSPG-INCLUDE-CONTENT:   replaced by the content enclosed in the <body></body> tags of page.html\n"
	print "DSPG-INCLUDE-PAGETITLE: replaced by Page title.\n"
	
end


################################################################################
# PageElement
###############################################################################
# class holding page element data
#
class PageElement
	attr_accessor :position,:title,:html,:headers
	def initialize(position,title,html,headers)
		@position = position
		@title = title
		@html = html
		@headers = headers
	end
end

###############################################################################
# FindLink
###############################################################################
# For each header that can be displayed, find where to link. 
# If the header does not have a corresponding page, search down the 
# hierarchy for the first available page.
# Returns "" is no page found down the hierarchy.
def FindLink(config,idx)
	link = nil
	(idx).upto(config.length-1) do |i|
		if link==nil && config[i].position>=config[idx].position
			link = config[i].html
		end
	end
	if link==nil
		link=""
	end
	return link
end


###############################################################################
# CreateTestPages
###############################################################################
# Create test pages only with the navigation hierarchy from the config file. 
#
# Create web pages with links and headers 

def CreateTestPages(config)
	config.each_index{|index|
		pe = config[index]
		next if pe.html==nil
		page=""	
	
		page << pe.title+"\n"
		page << "<BR>\n"
	
		pe.headers.each{|i|
			page << "<a href=\""+FindLink(config,i)+"\">"
			page << "<h" + (1+config[i].position).to_s + ">"
			if index == i
				page << "<i>"
			end
			page << config[i].title
			if index == i
				page << "</i>"
			end
			page << "</h" + (1+config[i].position).to_s + ">"
			page << "</a><br>\n"
		}
		#print page
		File.open("target/"+pe.html,"w") do |f|
			f << page
		end
	}
end


###############################################################################
# CreateSidebar
###############################################################################
# Create sidebar header strings with the navigation hierarchy from the config file for page index. 
#
# Create web pages with links and headers 

def CreateSidebar(config,index)
	pe = config[index]
	next if pe.html==nil
	#page="<div class=\"sidemenu\">\n"	
	page=""
	
	pe.headers.each{|i|
		
		# Div header
		
		page << "<p class=\"sh" + (1+config[i].position).to_s + "_"
		if index==i
			page << "active"
		else
			page << "inactive"
		end
		page << "\">"
		
		# a href and text
		
		page << "<a href=\""+FindLink(config,i)+"\">"
		page << config[i].title
		page << "</a></p>\n"
		
	}
	#page << "</div>\n"

	return page
end

###############################################################################
# ReadConfigFile
###############################################################################
# Reads the configuration file and returns the entries stored in an array of PageElement
#

def ReadConfigFile(cfile)
	config = Array.new
	
	linectr=0
	File.foreach(cfile) do |line|
		linectr+=1
		
		# Skip if the line is a comment (starting with #)
		# Note: this is actually overkill: we could strip! the line, and then check the first character.
		next if line =~ /^\s*#/
	
		# Remove spaces before and after	
		line.strip!
		
		# Skip if empty line
		next if line.length==0
	
		# At this stage, interpret the line. 
		# Format: [----]Page title[;html page]
		#print line, "\n"
		
		# Find position of first chacter different from '-'
		# pos is the number of dashes before the title.
		pos = line =~ /[^-]/
		if pos==nil
			print "Syntax error on line ",linectr,": missing title.\n"
			exit
		end
		
		# Obtain the substring w/o dashes
		line2 = line[pos..-1]
		
		# Split at the optional semicolumn.
		a = line2.split(';')
		if a.length>2
			print "Syntax error on line ",linectr,":extra ';'.\n"
			exit	
		end
	
		# Store for further use.
		pe = PageElement.new(pos,a[0],a[1],nil)
		config<<pe
	end
	return config
end

###############################################################################
# GetHTMLBody
###############################################################################
# Returns the content enclosed in the <body></body> tags. 
# If the tags do not exist returns passed string.
#
def GetHTMLBody(page)
	pos1 = page =~ /\<body\>/
	pos2 = page =~ /\<\/body\>/
	if pos1==nil || pos2==nil
		return page
	end
	return page[pos1+6...pos2]	
end

###############################################################################
# Main
###############################################################################
#

Banner()

if ARGV.length != 2 && ARGV.length!=3
	Syntax()
	exit
end

print "Source directory: ",ARGV[0],"\n"
print "Target directory: ",ARGV[1],"\n"


# Read the configuration file
config = ReadConfigFile(ARGV[0]+"/config")


#print "Number of pages: ",config.length,"\n"

# dump
#config.each{|pe| 
#	print "Position ",pe.position," title: '",pe.title,"' HTML: '",pe.html,"'\n"	
#	}
	
# Find the effective number of pages to generate: only those with a html file
#numpage=0
#config.each{|pe| 
#	if pe.html!=nil
#		numpage+=1
#	end
#	}
#print "Number of effective pages: ",numpage,"\n"

# Generate the headers displayed for each generated page

# iterate and do each page.
print "Reading configuration...\n"
config.each_index{|index| 
	# Skip entries w/o html files.
	pe = config[index]
	next if pe.html==nil
	
			
	#print "Generate '",pe.html,"' index ",index," pos ",pe.position,"\n"	
	
	# For page 'index' find which headers must appear. Store in toshow array.
	toshow = Array.new
	toshow<<index;
	
	# Search pages upwards from index. 
	# Display positions smaller or equal to the last position
	minpos = pe.position
	(index-1).downto(0) do |i|
		if config[i].position <= minpos
			toshow<<i
			minpos = config[i].position
		end
	end
	
	# Search pages downwards from index.
	# Display positions smaller or equal to the last position
	# Display positions higher by one than the current page as long as on the same subpage space.
	minpos = pe.position
	samesub=true				# keep track if going down the hierarchy of subpages
	(index+1).upto(config.length-1) do |i|
		if config[i].position==pe.position+1 && samesub
			toshow<<i
		end
		if config[i].position <= minpos
			toshow<<i
			minpos = config[i].position
			samesub=false						# went up the hierarchy, not the same subpage space
		end
	end
	toshow.sort!
	config[index].headers=toshow;
	
	#print "For page ",pe.title," showing headers: "
	#toshow.each{|i| print i," "}
	#print "\n"
}

# dump
#config.each{|pe| 
#	next if pe.html==nil
#	print "**********************\n"
#	print "(",pe.title,")\n"
#	pe.headers.each{|i|
#		print "\t","\t"*config[i].position,config[i].title," \t ",config[i].html," \t ",FindLink(config,i),"\n"
#	}
#}


# Test pages
#CreateTestPages(config)

# Load the template
print "Reading template...\n"
template = File.read(ARGV[0]+"/template")
	
# Generate the pages
print "Writing pages...\n"
numpage=0
config.each_index{|index|
	next if config[index].html==nil
	numpage += 1
	
	# load the page from the source directory
	contentall = File.read(ARGV[0]+"/"+config[index].html)
	content = GetHTMLBody(contentall)
	
	# Load the content of the sidebar
	sidebar = CreateSidebar(config,index)
	
	# Inject into the template
	page = template
	page = page.sub("DSPG-INCLUDE-SIDEMENU",sidebar)
	page = page.sub("DSPG-INCLUDE-CONTENT",content)
	page = page.sub("DSPG-INCLUDE-PAGETITLE",config[index].title)
	
	
	File.open(ARGV[1]+"/"+config[index].html,"w") do |f|
		f << page
	end
}
		
print "Wrote ", numpage," pages.\n"

# copy all the content
if ARGV.length==3 && ARGV[2].upcase=="COPY"
	print "Copying remaining content of source to target...\n"
	
	# Files NOT to copy
	skip = Array.new
	skip << "."
	skip << ".."
	skip << "template"
	skip << "config"
	config.each{|pe| 
		next if pe.html==nil
		skip << pe.html
	}
	Dir.foreach(ARGV[0]+"/") {|x| 
		next if skip.include?(x)
		#print "copying ",x,"\n"
		cp_r(ARGV[0]+"/"+x,ARGV[1])
	}	
end

print "Done.\n"